/*
 * File         : BV_Msg_Header.cpp
 * Author       : Tomasz Stachowiak
 * Created      : Wed May 20 21:25:10 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <unistd.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <BVN/SOCK_Stream.h>
#include "BV_Msg_Header.h"
#include "BV_Session_Control.h"
#include "BV.h"

BV_Msg_Header::BV_Msg_Header() : size_(0) 
{
  sender_.handle = 0;
  BV_Session_Control *ctrl = BV_Session_Control::instance();
  Options *opt = ctrl->get_options();
  ctrl->destroy();
  strcpy(sender_.user, opt->local_name);
  sender_.host = gethostid();
}

int BV_Msg_Header::recv(SOCK_Stream *stream)
{ 
  BVN_DEBUG("BV_Msg_Header::recv\n");

  int n = stream->recv_n( buffer_, MSG_HEADER_LENGTH);
  if(n == -1)
    {
      BVN_ERROR("BV_Msg_Header::recv\n");
      return -1;
    }
  if(n == 0)
    return 0;
  
  tag_ = buffer_[0];
  session_ = buffer_[1];

  sender_.handle = buffer_[2];
  strcpy(sender_.user, &buffer_[3]);
  memcpy(&sender_.host, &buffer_[23], sizeof(int));

  receiver_.handle = buffer_[27];
  strcpy(receiver_.user, &buffer_[28]);
  memcpy(&receiver_.host, &buffer_[48], sizeof(int));
  
  memcpy(&size_, &buffer_[52], sizeof(int));
  size_ = ntohl(size_);

  char host[50];
  struct in_addr addr;
  addr.s_addr = sender_.host;
  strcpy(host, inet_ntoa(addr));
  printf("BV_Msg_Header: received message of type %d from %s, size %d\n", tag_, host, size_);

  return 1; 
}

int BV_Msg_Header::send(SOCK_Stream *stream)
{ 
  buffer_[0] = tag_;
  buffer_[1] = session_;

  buffer_[2] = sender_.handle;
  strcpy(&buffer_[3], sender_.user);
  memcpy(&buffer_[23], &sender_.host, sizeof(int));

  buffer_[27] = receiver_.handle;
  strcpy(&buffer_[28], receiver_.user);
  memcpy(&buffer_[48], &receiver_.host, sizeof(int));
  
  size_ = htonl(size_);
  memcpy(&buffer_[52], &size_, sizeof(int));
  size_ = ntohl(size_);

  return stream->send_n( buffer_, MSG_HEADER_LENGTH); 
}

unsigned char BV_Msg_Header::get_tag() 
{ 
  return tag_; 
}

unsigned char BV_Msg_Header::get_session() 
{ 
  return session_; 
}

void BV_Msg_Header::get_sender(Participant *part) 
{ 
  memcpy(part, &sender_, sizeof(struct _participant)); 
}

void BV_Msg_Header::set_sender_id(unsigned char id)
{
  sender_.handle = id;
}

void BV_Msg_Header::get_receiver(Participant *part) 
{ 
  memcpy(part, &receiver_, sizeof(struct _participant)); 
}

int BV_Msg_Header::get_size() 
{ 
  return size_; 
}

void BV_Msg_Header::set_tag(unsigned char tag) 
{ 
  tag_ = tag; 
}

void BV_Msg_Header::set_session(unsigned char session) 
{ 
  session_ = session;
}

void BV_Msg_Header::set_sender(Participant *part) 
{ 
  memcpy(&sender_, part, sizeof(struct _participant)); 
}

void BV_Msg_Header::set_receiver(Participant *part)
{ 
  memcpy(&receiver_, part, sizeof(struct _participant)); 
}

void BV_Msg_Header::set_size(int size) 
{ 
  size_ = size;
}
